{-# LANGUAGE OverloadedStrings #-}

module Data.Time.Calendar where

import Data.Time.Segment.Main (now, timesBetween, Granularity(..))
import Control.Lens ((^.))
import Data.ByteString.Lazy (ByteString)
import Data.Default as D
import Data.Either
import Data.List as List
import Data.Map as Map
import Data.Maybe (catMaybes)
import Data.Set (Set)
import Data.Set as Set
import Data.String (String)
import Data.Time as Time
import Network.Wreq as W
import Safe (headMay)
import Text.HTML.TagSoup as Soup
import Text.ICalendar.Parser as P
import Text.ICalendar.Types as Cal

-- import Data.ByteString.Lazy (toStrict)
-- import Data.Text.Encoding (decodeUtf8)

todo = undefined

type Calendar = ByteString
type CalendarUrl = String

data Meeting = Meeting { meetingStart :: UTCTime
                       , meetingEnd   :: UTCTime
                       }
-- Target Function
-- Let be a duration for a meeting
-- Let be a Set of remote calendar urls
-- The result of easyDating is a Map with Dates and its Participants
-- The result can be ordered according to any given policy
easyDating :: Duration -> Set CalendarUrl -> IO (Map Meeting (Set CalendarUrl))
easyDating = undefined

----------------------------------------------------------------------
-- Main Types
-- TODO
-- 2 types of Calendar
-- CalendarOfAvailability | CalendarOfOccupation
-- We suppose Calendar of availability for now
data Agenda = Availability Calendar | Absence Calendar

buildCalendar :: Agenda -> Agenda
buildCalendar (Availability cal) = Availability cal
buildCalendar (Absence _) = todo

-----------------------------------------------------------------
toMeeting :: (DTStart, Either DTEnd DurationProp) -> Meeting
toMeeting = undefined

getDurations :: Maybe VCalendar
             -> [(DTStart, Either DTEnd DurationProp)]
getDurations Nothing = []
getDurations (Just vCal) = catMaybes
                         $ Prelude.map (\e -> (,) <$> Cal.veDTStart e <*> Cal.veDTEndDuration e)
                         -- if there is either no Start or no End, then the duration is wrong
                         -- we remove it accordingly
                         $ Map.elems
                         $ Cal.vcEvents vCal

untilThis :: DTStart
         -> [(DTStart, Either DTEnd DurationProp)]
         -> [(DTStart, Either DTEnd DurationProp)]
untilThis d xs = List.filter (\(s,_) -> d >= s) xs

-- Now depends on IO
-- Then it is in a IO Monad
untilNow ::    [(DTStart, Either DTEnd DurationProp)]
         -> IO [(DTStart, Either DTEnd DurationProp)]
untilNow xs = do
  n <- now
  pure $ untilThis (DTStartDateTime (UTCDateTime n) (OtherParams Set.empty)) xs


splitDuration :: (Date,Date) -> Duration -> [(Date,Date)]
splitDuration = undefined

---------------------------------------------------------------
-- Tools to Get VCalendars
-- Local VCalendar
vCalendar :: FilePath -> IO (Maybe VCalendar)
vCalendar filePath = do
  vcal <- P.parseICalendarFile  D.def filePath
  (cals, _) <- case vcal of
    Left err  -> error err
    Right res -> pure res
  pure $ headMay cals


-- Remote VCalendar
-- Open Url for tests
myCalendarUrl :: String
myCalendarUrl = "https://delanoe.org/nextcloud/apps/calendar/p/LwNxPHnnBRtMMqwB"

myCalendar :: IO ByteString
myCalendar = do
  cal <- W.get myCalendarUrl
  pure $ cal ^. W.responseBody

remoteCalendar :: CalendarUrl -> IO (Maybe VCalendar)
remoteCalendar = undefined
-- $ P.parseICalendar D.def "/tmp/errors" cal'
---------------------------------------------------------------
